@@ -3,7 +3,7 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018 Massachusetts Institute of Technology
+    2017, 2018, 2019 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -159,6 +159,12 @@ Initialized from the SHELL environment variable."
 	   (let ((channel (subprocess-input-channel subprocess)))
 	     (if channel
 		 (channel-nonblocking channel)))
+           (let ((fix-port
+                  (lambda (port)
+                    (if (and port (port/supports-coding? port))
+                        (port/set-coding port 'iso-8859-1)))))
+             (fix-port (subprocess-input-port subprocess))
+             (fix-port (subprocess-output-port subprocess)))
 	   (set-process-status-registration!
 	    process
 	    (register-subprocess-event
