@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -25,13 +26,20 @@ USA.
 
 ;;;; IMAIL mail reader: compilation
 
-(load-option 'CREF)
-(load-option 'SOS)
-(load-option '*PARSER)
+(load-option 'cref)
 (with-working-directory-pathname (directory-pathname (current-load-pathname))
   (lambda ()
+    (with-working-directory-pathname (merge-pathnames "../sos")
+      (lambda ()
+	(load "load")))
+    (with-working-directory-pathname (merge-pathnames "../star-parser")
+      (lambda ()
+	(load "load")))
+    (with-working-directory-pathname (merge-pathnames "../edwin")
+      (lambda ()
+	(load "make")))
     (for-each (lambda (filename)
-		(compile-file filename '() (->environment '(EDWIN))))
+		(compile-file filename '() (->environment '(edwin))))
 	      '("imail-browser"
 		"imail-core"
 		"imail-file"
@@ -44,4 +52,4 @@ USA.
 		"imail-util"
 		"imap-response"
 		"imap-syntax"))
-    (cref/generate-constructors "imail" 'ALL)))
\ No newline at end of file
+    (cref/generate-constructors "imail")))
\ No newline at end of file
